@@ -1,6 +1,6 @@
 # E2::Node
 # Jose M. Weeks <jose@joseweeks.com>
-# 18 June 2003
+# 11 July 2003
 #
 # See bottom for pod documentation.
 
@@ -14,7 +14,7 @@ use Carp;
 use E2::Interface;
 
 our @ISA = "E2::Interface";
-our $VERSION = "0.32";
+our $VERSION = "0.33";
 our $DEBUG; *DEBUG = *E2::Interface::DEBUG;
 
 # Prototypes
@@ -72,60 +72,65 @@ sub autodetect {
 	
 	warn "E2::Node::autodetect\n"	if $DEBUG > 1;
 	
-	bless $self, 'E2::Node';
+	bless $self;
 	return 1;
 }
 
 sub load {
-	my $self  = shift or croak "Usage: load E2NODE, TITLE [, TYPE]";
-	my $title = shift or croak "Usage: load E2NODE, TITLE [, TYPE]";
+	my $self  = shift or croak "Usage: load E2NODE, TITLE [, TYPE] [, SOFTLINK ]";
+	my $title = shift or croak "Usage: load E2NODE, TITLE [, TYPE] [, SOFTLINK ]";
 	my $type  = shift || $self->type_as_string;
+	my $softlink = shift||0;
 
 	warn "E2::Node::load\n"		if $DEBUG > 1;
 
 	warn "Loading node $title"	if $DEBUG > 2;
 
+	$softlink = $self->node_id	if $softlink == -1;
+
 	my %opt;
 
 	$opt{node}	  = $title;
 	$opt{displaytype} = 'xmltrue';
 	$opt{type}	  = $type	if $type;
+	$opt{lastnode_id} = $softlink	if $softlink;
 
 	return $self->thread_then(
 		[ \&E2::Interface::process_request, $self, %opt ],
 	sub {
 		my $r = shift;
-		return $self->load_from_xml( $r, $type );
+		return $self->load_from_xml( $r );
 	});
 }
 
 sub load_by_id {
-	my $self = shift or croak "Usage: load_by_id E2NODE, NODE_ID [, TYPE]";
-	my $id   = shift or croak "Usage: load_by_id E2NODE, NODE_ID [, TYPE]";
-	my $type = shift || $self->type_as_string;
-
+	my $self = shift or croak "Usage: load_by_id E2NODE, NODE_ID [, SOFTLINK ]";
+	my $id   = shift or croak "Usage: load_by_id E2NODE, NODE_ID [, SOFTLINK ]";
+	my $softlink = shift||0;
+	
 	warn "E2::Node::load_by_id\n"	if $DEBUG > 1;
 
 	warn "Loading node_id $id"	if $DEBUG > 2;
 
+	$softlink = $self->node_id	if $softlink == -1;
+
 	my %opt;
 
-	$opt{node_id}		= $id;
-	$opt{displaytype}	= 'xmltrue';
-	$opt{type}		= $type		if $type;
-
+	$opt{node_id}	  = $id;
+	$opt{displaytype} = 'xmltrue';
+	$opt{lastnode_id} = $softlink	if $softlink;
+	
 	return $self->thread_then( [ \&E2::Interface::process_request, $self, %opt ],
 	sub {
 		my $r = shift;
-		return $self->load_from_xml( $r, $type );
+		return $self->load_from_xml( $r );
 	});
 }
 
 
 sub load_from_xml {
-	my $self = shift or croak "Usage: load_from_xml E2NODE, XML_STRING [, TYPE ]";
-	my $xml	 = shift or croak "Usage: load_from_xml E2NODE, XML_STRING [, TYPE ]";
-	my $type = shift || $self->type_as_string;
+	my $self = shift or croak "Usage: load_from_xml E2NODE, XML_STRING";
+	my $xml	 = shift or croak "Usage: load_from_xml E2NODE, XML_STRING";
 
 	warn "E2::Node::load_from_xml\n"	if $DEBUG > 1;
 
@@ -149,20 +154,23 @@ sub load_from_xml {
 	}
 
 	# If we hit a search page then the node doesn't exist
+	# FIXME: search page
 
 	if( ! ($xml =~ /<node [^>]*node_id="(.*?)"/s) ) {
 		croak "Invalid document";
 	}
 
-	if( $1 == 1140332 && 				# Search node_id
-	    (!$type || lc($type) ne 'superdoc') ) {
+	if( $1 == 1140332 ) {
+	#&& 				# Search node_id
+	#    (!$type || lc($type) ne 'superdoc') ) {
 	    	warn "Hit search superdoc"	if $DEBUG;
+		$self->clear;
 		return undef;
 	}
 
-	if( $type && lc($type) ne lc($verified_type) ) {
-		return undef;
-	}
+	#if( $type && lc($type) ne lc($verified_type) ) {
+	#	return undef;
+	#}
 
 	# Now, if we are an E2::Node, figure out what type of node
 	# the XML is describing and bless $self into that class.
@@ -372,11 +380,11 @@ These methods return, respectively, the title of the node, the node_id, the auth
 
 =item $node-E<gt>exists
 
-This method returns a boolean value in answer to the question "Does this node exist?" This can be used to test whether or not a node has been loaded correctly.
+Boolean: "Does this node exist?"
 
-=item $node-E<gt>load TITLE [, TYPE ]
+=item $node-E<gt>load TITLE [, TYPE ] [, SOFTLINK ]
 
-=item $node-E<gt>load_by_id NODE_ID
+=item $node-E<gt>load_by_id NODE_ID [, SOFTLINK ]
 
 =item $node-E<gt>load_from_xml XML_STRING
 
@@ -386,13 +394,19 @@ C<load_from_xml> expects to be passed an XML string of the type generated by a q
  
 C<load> and C<load_by_id> fetch the appropriate page from everything2.com. For C<load>, if TYPE is specified, it fetches the node of that type. If no appropriate node of that type exists, they return C<undef>. Otherwise, they return true.
 
+If SOFTLINK is specified (as a node_id), $node attempts to create a softlink between SOFTLINK and the node it is loading. If SOFTLINK is -1, $node attempts to create a softlink between the previously-loaded node and the node it is loading.
+
+No softlinks will be created unless SOFTLINK is specified.
+
 If the object that's doing the C<load>ing is of this class, rather than one of its descendants, the C<load> methods will attempt to determine from the XML the type of node they were passed, and then re-C<bless> the current object into that class. These are the classes an E2::Node object will be re-C<bless>ed into based on node type:
 
 	e2node		=> E2::E2Node
 	writeup		=> E2::E2Node	# NOTE: Not E2::Writeup
 	user		=> E2::User
 	usergroup	=> E2::Usergroup
+	room		=> E2::Room
 	superdoc	=> E2::Superdoc
+	superdocnolinks	=> E2::Superdoc
 
 And here's an example:
 
@@ -409,7 +423,7 @@ Note: once an object has been re-C<bless>ed, it is a member of the new class, an
 
 	# (continued from above)
 
-	$node->load( "nate", "user" ); # throws 'Wrong node type:' exception.
+	$node->load( "nate", "user" ); # throws 'Wrong node type:'
 
 Once the object has been re-C<bless>ed, if we wish to autodetect node type again, we must call C<$node-E<gt>autodetect>.
 
@@ -441,7 +455,8 @@ Example use:
 		chomp $type;
 
 		$node->autodetect;
-		$node->load( $title, $type ) or die "Unable to load node";
+		$node->load( $title, $type )
+			or die "Unable to load node";
 
 		&display_node( $node );
 	}
@@ -462,8 +477,8 @@ Example use:
 			# ...
 		} 
 
-		# There should be a bunch of elsif ... statements here,
-		# each displaying a specific type of node
+		# There should be a bunch of elsif ... statements
+		# here, each displaying a specific type of node
 		# ...
 	}
 
